<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;
/**
 * Typecho 导航菜单插件
 * 
 * @package TeMenu 
 * @author 绛木子
 * @version 2.0.0
 * @link http://lixianhua.com
 */
class TeMenu_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        // 增加导航菜单管理
		Helper::addPanel(3, 'TeMenu/panel.php', '菜单', '管理菜单', 'administrator');
        // 增加action
		Helper::addAction('TeMenu','TeMenu_Action');
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate(){
        Helper::removeAction('TeMenu');
        Helper::removePanel(3,'TeMenu/panel.php');
    }
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        // 存储导航菜单数据
		$json = '{"header":{"menu":[{"target":"0","icon":"","type":"custom","name":"\u9996\u9875","id":"{siteUrl}"}],"title":"\u5bfc\u822a\u83dc\u5355"}}';
		$json = str_replace('{siteUrl}', Helper::options()->siteUrl,$json);
		$navigation = new Typecho_Widget_Helper_Form_Element_Textarea('navigation', NULL, $json, _t('菜单数据'), _t('菜单数据，请通过菜单管理工具进行编辑'));
		$form->addInput($navigation);
		$locations = new Typecho_Widget_Helper_Form_Element_Textarea('locations', NULL, '[]', _t('主题注册的菜单位置'), _t('主题注册的菜单位置数据，请勿在此做任何修改'));
		$form->addInput($locations);
	}
	
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
	
	/**
     * 主题注册支持的菜单位置
     * 
     * @access public
	 * @param array $locations 菜单位置标识 array('locationSlug'=>'locationName')
	 * @param string $theme 主题标识
     * @return void
     */
	public static function register(array $locations, $theme){
		$defaultLocations = Helper::options()->plugin('TeMenu')->locations;
		if(!empty($defaultLocations)){
			$defaultLocations = json_decode($defaultLocations, true);
			if(isset($defaultLocations[$theme])){
				return false;
			}
		}
		foreach($locations as $location=>$locationName){
			$locations[$location] = array(
				'title' => $locationName,
				'navigation' => '',
			);
		}
		$settings = array(
			'navigation' => Helper::options()->plugin('TeMenu')->navigation,
			'locations' => json_encode(array($theme => $locations))
		);
		Helper::configPlugin('TeMenu', $settings);
	}

	public static function getNavigation(){
		return json_decode(Helper::options()->plugin('TeMenu')->navigation,true);
	}

	public static function getLocations(){
		$locations = json_decode(Helper::options()->plugin('TeMenu')->locations, true);
		$theme = Helper::options()->theme;
		return $locations && isset($locations[$theme]) ? $locations[$theme] : false;
	}

	public static function getNavigationLocationTitles($slug){
		$navigation = self::getNavigation();
		$locations = self::getLocations();
		if(!isset($navigation[$slug]) || empty($locations)){
			return '';
		}
		$used = array();
		foreach($locations as $location=>$locationVal){
			if($slug == $locationVal['navigation']){
				$used[] = $locationVal['title'];
			}
		}
		return implode(',', $used);
	}

	public static function menu($location){
		$navigation = self::getNavigation();
		$locations = self::getLocations();
		$menu = array();
		if(isset($locations[$location]) && !empty($locations[$location]['navigation']) && isset($navigation[ $locations[$location]['navigation'] ])){
			$menu = $navigation[ $locations[$location]['navigation'] ]['menu'];
		}
		if($menu){
			$menu = self::parseMenu($menu);
		}
		return $menu;
	}

    /**
	  * 显示菜单
	  * @param string $slug 菜单名称
	  * @param string $options 菜单参数
	  * 
	  * item:菜单元素
	  * itemTag:菜单元素的标签
	  * childTag:子菜单标签
	  * childClass:子菜单css类
	  * caret:是否显示子菜单标记
	  * caretDiv:子菜单标记代码
	  */
	public static function show($location, $options=''){
		$menu = self::menu($location);
		if(empty($menu)){
			return;
		}
		$options = new Typecho_Config($options);
		$options->setDefault(array(
			'wrapTag'=>'ul',
			'wrapClass'=>'',
			'itemTag'=>'li',
			'itemClass'=>'',
			'item'=>'<a {current} href="{url}" {target}>{icon} {name}</a>',
			'childTag'=>'ul',
			'childClass'=>'child',
			'current' => 'current',
			'caret'	=> '',
		));
		$html = '';
		$func = strtolower($location).'MenusHtml';
		if (function_exists($func)) {
            $html = $func($menu, $options);
        }else{
			if($options->wrapTag){
				$html = '<' . $options->wrapTag . (empty($options->wrapClass)
						? '' : ' class="' . $options->wrapClass . '"') . '>';
			}
			
			$html .= self::outputMenu($menu, $options);
			if($options->wrapTag){
				$html .= '</' . $options->wrapTag . '>';
			}
		}
		echo $html;
	}
	// 解析菜单数据
	private static function outputMenu($menu, &$options, $isChild = false){
		$html = '';
		foreach($menu as $k=>$v){
			$caret = $isChild ? '' : $options->caret;
			$v['target'] = isset($v['target']) && $v['target'] ? 'target="_blank"' : '';

			$v['icon'] = isset($v['icon']) ? '<i class="'.$v['icon'].'"></i>' : '';
			$v['current'] && $v['current'] = $options->current;
			
			$html .= $options->itemTag ? '<'. $options->itemTag . (empty($options->itemClass)
							? '' : ' class="' . str_replace('{current}', $v['current'], $options->itemClass) . '"') . '>' : '';
			
			$html .= str_replace(array('{url}', '{name}', '{current}', '{icon}', '{target}', '{caret}'),
				array($v['url'], $v['name'], $v['current'], $v['icon'], $v['target'], $caret), $options->item);
			
			if(isset($v['children'])){
				if($options->childTag){
					$html .= '<' . $options->childTag . (empty($options->childClass)
							? '' : ' class="' . $options->childClass . '"') . '>';
				}
				
				$html .= self::outputMenu($v['children'], $options, true);
				
				$html .= $options->childTag ? '</'. $options->childTag . '>' : '';
			}
			$html .= $options->itemTag ? '</'. $options->itemTag . '>' : '';
		}
		return $html;
	}

	private static function parseMenu($menu){
		$archive = Typecho_Widget::widget('Widget_Archive');
		foreach($menu as $k=>&$v){
			if($v['type']=='category'){
				$category = self::getCategory($v['id']);
				$v['url'] = $category['permalink'];
				$v['slug'] = $category['slug'];
				$v['current'] = $archive->is('category', $category['slug']);
			}elseif($v['type']=='page'){
				$page = self::getPage($v['id']);
				$v['url'] = $page['permalink'];
				$v['slug'] = $page['slug'];
				$v['current'] = $archive->is('page',$page['slug']);
			}else{
				$url = $archive->request->getRequestUrl();
				$v['url'] = $v['id'];
				if(false === strpos($v['url'], 'http') && '#' != $v['url'] && false === strpos($v['url'], 'javascript:')){
					$v['url'] = Typecho_Common::url($v['url'], Helper::options()->index);
				}
				$v['slug'] = '';
				$v['current'] = $url === $v['url'];
			}
			if(isset($v['children'])){
				$v['children'] = self::parseMenu($v['children']);
			}
		}
		return $menu;
	}

    // 获取单个页面
	public static function getPage($cid){
		return Typecho_Widget::widget('Widget_Page_Query')->getPage($cid);
	}
	// 获取分类
	public static function getCategory($mid){
		return Typecho_Widget::widget('Widget_Metas_Category_List')->getCategory($mid);
	}

    // 后台显示菜单操作列表
	public static function showNavigationDrag($navigation){
		$html = '';
		if(empty($navigation)) return '';
		
		$html .='<ol class="dd-list">';
		foreach($navigation as $k=>$v){
			$v['target'] = isset($v['target']) && $v['target'] ? 1 : 0;
			$v['icon'] = isset($v['icon']) ? $v['icon'] : '';

			$html .= '<li class="dd-item" data-id="'.$v['id'].'" data-name="'.$v['name'].'" data-type="'.$v['type'].'" data-icon="'.htmlspecialchars($v['icon']).'" data-target="'.$v['target'].'">';
			$html .= '<div class="dd-handle">Drag</div><div class="dd-content">'.$v['name'].'</div><div class="dd-setting">Setting</div><div class="dd-remove">Remove</div>';
			$html .= '<div class="dd-setdiv"><label>名称</label><input class="w-100 text dd-name" type="text" value="'.$v['name'].'" />';
			if($v['type']=='custom'){
				$html .= '<label>链接</label><input class="w-100 text dd-url" type="text" value="'.$v['id'].'" />';
			}
			$html .= '<label>新窗口打开</label><input class="text dd-target" type="checkbox" value="1" '. ($v['target'] ? 'checked' : '') .'/>用新窗口打开链接';
			$html .= '<label>图标样式</label><input class="w-100 text dd-icon" type="text" value="'.htmlspecialchars($v['icon']).'"/>';
			$html .= '</div>';
			if(isset($v['children'])){
				$html .= self::showNavigationDrag($v['children']);
			}
			$html .= '</li>';
		}
		$html .= '</ol>';
		return $html;
	}
}
class Widget_Page_Query extends Widget_Abstract_Contents
{
	 /**
     * 所有页面哈希表 
     * 
     * @var array
     * @access private
     */
    private $_map = array();
    /**
     * 执行函数
     *
     * @access public
     * @return void
     */
    public function execute()
    {
        $select = $this->select()->where('table.contents.type = ?', 'page')
        ->where('table.contents.created < ?', $this->options->gmtTime)
        ->order('table.contents.order', Typecho_Db::SORT_ASC);

        //去掉自定义首页
        $frontPage = explode(':', $this->options->frontPage);
        if (2 == count($frontPage) && 'page' == $frontPage[0]) {
            $select->where('table.contents.cid <> ?', $frontPage[1]);
        }

        $rs = $this->db->fetchAll($select, array($this, 'push'));
		foreach($rs as $v){
			$this->_map[$v['cid']] = $v;
		}
    }
	/**
     * 获取单个页面
     * 
     * @param integer $cid 
     * @access public
     * @return mixed
     */
    public function getPage($cid)
    {
        return isset($this->_map[$cid]) ? $this->_map[$cid] : NULL;
    }
}